"""
This script shows how to use a sequence alignment to predict binding site 
residues in one structure from residues close to a ligand in another
"""
from ost.bindings import msms
from ost.bindings import tmtools

structure_wi_lig = io.LoadPDB('data/structure-with-ligand.pdb')
structure_no_lig = io.LoadPDB('data/structure-without-ligand.pdb')
print tmtools.TMAlign(structure_no_lig, structure_wi_lig)

for res in structure_wi_lig.residues:
  res.SetFloatProp('close',0.0)
for res in structure_wi_lig.Select('4.0 <> [rname=RVP]').residues:
  res.SetBoolProp('close_to_ligand', True)
  res.SetFloatProp('close',1.0)


# get the sequence for both structures
ligand_seq = seq.SequenceFromChain('ligand', structure_wi_lig.chains[0])
no_lig_seq = seq.SequenceFromChain('no lig', structure_no_lig.chains[0])
# align the two sequences using a global alignment algorithm with the 
# BLOSUM62 substitution matrix and default gap extension and opening 
# penalties. Global align returns a list of global alignments, but we
# only care about the first one...
aln = seq.alg.GlobalAlign(ligand_seq, no_lig_seq, seq.alg.BLOSUM62)[0]

# print the alignment, to make sure the alignment is reasonable
print aln

# the alignment essentially defines a mapping of residues in the first
# and the second structure. We will use GetMatchingBackboneViews to
# obtain two entity views which contain the corresponding residues
aln_wi_lig, aln_no_lig = aln.GetMatchingBackboneViews()

# iterate over the residue pairs and print residue names of residues
# which are part of the "predicted" binding site
print 'predicted binding site'
for lig_res, no_lig_res in zip(aln_wi_lig.residues, aln_no_lig.residues):
  if lig_res.HasProp('close_to_ligand'):
    print no_lig_res
    no_lig_res.SetFloatProp('close',1.0)
  else:
    no_lig_res.SetFloatProp('close',0.0)
    

scene=gfx.Scene()

#scene.Add(gfx.Entity('wi',structure_wi_lig))
#scene.Add(gfx.Entity('no',structure_no_lig))

pwi = structure_wi_lig.Select('protein=True')
pno = structure_no_lig.Select('protein=True')
swi = msms.CalculateSurface(pwi, density=4)[0]
swi.Attach(pwi, 5)
swi_g = gfx.Surface('swi',swi)
sno = msms.CalculateSurface(pno, density=4)[0]
sno.Attach(pno, 5)
sno_g = gfx.Surface('sno',sno)
scene.Add(swi_g)
scene.Add(sno_g)
pwi_g = gfx.Entity('pwi',pwi)
lwi_g = gfx.Entity('lwi',structure_wi_lig.Select('rname=RVP'))
pno_g = gfx.Entity('pno',pno)


gradient=gfx.Gradient()
gradient.SetColorAt(0, gfx.WHITE)
gradient.SetColorAt(0.5, gfx.WHITE)
gradient.SetColorAt(1.0, gfx.Color(0.45,0.45,1.0))

gradient2=gfx.Gradient()
gradient2.SetColorAt(0, gfx.WHITE)
gradient2.SetColorAt(0.5, gfx.WHITE)
gradient2.SetColorAt(1.0, gfx.Color(1.0,0.4,0.4))

sno_g.ColorBy('close', gradient2, -1, 1, mol.Prop.Level.RESIDUE)
pno_g.ColorBy('close', gradient2, -1, 1, mol.Prop.Level.RESIDUE)
swi_g.ColorBy('close', gradient, -1, 1, mol.Prop.Level.RESIDUE)
pwi_g.ColorBy('close', gradient, -1, 1, mol.Prop.Level.RESIDUE)
lwi_g.SetRenderMode(gfx.CUSTOM)
lwi_g.SetColor(gfx.GREEN, 'ele=C')


lwi_g.SetOutline(True)
pwi_g.SetOutline(True)
pno_g.SetOutline(True)
swi_g.SetOutline(True)
sno_g.SetOutline(True)

lwi_g.SetOutlineMode(3)
pwi_g.SetOutlineMode(3)
pno_g.SetOutlineMode(3)
swi_g.SetOutlineMode(3)
sno_g.SetOutlineMode(3)

scene.Add(lwi_g)
#scene.Add(pwi_g)
#scene.Add(pno_g)

gfx.Shader.Instance().Activate('hemilight')
scene.bg=gfx.WHITE

scene.AutoAutoslab(False)
scene.rtc=geom.Mat4(0.320595, 0.129266, -0.93835, 7.73379, 0.107576, 0.979259, 0.171657, -48.2177, 0.941082, -0.155978, 0.300041, 5.58721, 0, 0, -100, 1)
scene.near=76.0
scene.far=105.0

scene.SetAmbientOcclusion(True)
scene.SetAmbientOcclusionMode(3)


scene.StartOffscreenMode(1800, 1700)
scene.InitGL(True)
gfx.Shader.Instance().Activate('hemilight')
sno_g.Hide()
scene.Export('with.png')
swi_g.Hide()
sno_g.Show()
lwi_g.Hide()
scene.Export('no.png')
scene.StopOffscreenMode()



